/*****************************************************************
 * CAUTION CAUTION CATION
 * Refrain from having the arduino be hooked up to the flight controller while it is powered by USB (via the computer)
 * If code is to be uploaded first detach 5V pin from the arduino to the flightcontroller
 * 
 * PINS
 * Digital 11: PPM signal to A1 on the flight controller
 * Digital 2: XBee white wire 
 * GND: Flight controller ground and Xbee black wire
 * 5V: Vin of the flight controller
 * 3.3V: Red wire of the XBee
 */

//////////////////////CONFIGURATION///////////////////////////////
#define chanel_number 8  //set the number of chanels
#define default_servo_value 1500  //set the default servo value
#define PPM_FrLen 22500  //set the PPM frame length in microseconds (1ms = 1000µs)
#define PPM_PulseLen 300  //set the pulse length
#define onState 0  //set polarity of the pulses: 1 is positive, 0 is negative
#define sigPin 5   //set PPM signal output pin on the arduino. This pin foes to A1 on the flight controller
//////////////////////////////////////////////////////////////////

//Xbee setup
#include <SoftwareSerial.h>
#include <String.h>
//SoftwareSerial XBee(12, 13); //connect XBee with the three wires. Black to ground, red to 3.3V on arduino, and white to arduino digital pin 2.

/*this array holds the servo values for the ppm signal
 change theese values in your code (usually servo values move between 1000 and 2000)*/
int ppm[chanel_number];
int incomingData[7];
int previousStates[4];
int dataSize = 7;
int numStates = 4;
int throttleIncrement = 20;

void setup(){  
  //initiallize default ppm values
  for(int i=0; i<chanel_number; i++){
    ppm[i] = default_servo_value;
  }

  pinMode(sigPin, OUTPUT);
  digitalWrite(sigPin, !onState);  //set the PPM signal pin to the default state (off) 

  //the code from here until Serial.begin is the register setup for creatin ppm.
  cli();
  TCCR1A = 0; // set entire TCCR1 register to 0
  TCCR1B = 0;
  
  OCR1A = 10000;  // compare match register, change this
  TCCR1B |= (1 << WGM12);  // turn on CTC mode
  TCCR1B |= (1 << CS11);  // 8 prescaler: 0.5 microseconds at 16mhz
  TIMSK1 |= (1 << OCIE1A); // enable timer compare interrupt
  sei();

  for(int i = 0; i < numStates; i++){
    previousStates[i] = 0;
  }
  
  Serial.begin(9600);
}

void loop(){   
  if(Serial.available() > 6){
    for(int i = 0; i < dataSize; i++){
      incomingData[i] = Serial.read();
    }
    ppm[2] = map(incomingData[0], 0, 256, 1000, 1500);
    ppm[0] = map(incomingData[1], 0, 256, 1000, 2000);
    ppm[1] = map(incomingData[2], 0, 256, 1000, 2000);
    if(incomingData[3] == 1 && previousStates[0] == 0){  //if the first byte (the arm disarm byte) is 1, meaning the copter needs to armed, set the ppm values (throttle and yaw) that arm the copter. This is equivalent to holding the joystick to the bottom right (look up youtube
      armCopter();
    }
    else if(incomingData[3] == 0 && previousStates[0] == 1){    //if the copter needs to be disarmed then set the corresponding ppm values to disarm it (again look at youtube video at how it gets disarmed)
      disarmCopter();
    }
    for(int i = 0; i < numStates; i++){
      previousStates[i] = incomingData[i+3];
    }
    for(int i = 0; i < 3; i++){
      ppm [5+i] = 1100 + incomingData[i+4]*800;
    }
  }
}

void disarmCopter(){
    ppm[2] = 1000;
    ppm[3] = 1000;
    delay(1000);
    ppm[3] = 1500;
}

void armCopter(){
    ppm[2] = 1000;                   //videos about arming multiiWii and you will understand
    ppm[3] = 2000;
    delay(1000);
    ppm[3] = 1500;
}

ISR(TIMER1_COMPA_vect){  //leave this alone
  static boolean state = true;
  
  TCNT1 = 0;
  
  if(state) {  //start pulse
    digitalWrite(sigPin, onState);
    OCR1A = PPM_PulseLen * 2;
    state = false;
  }
  else{  //end pulse and calculate when to start the next pulse
    static byte cur_chan_numb;
    static unsigned int calc_rest;
  
    digitalWrite(sigPin, !onState);
    state = true;

    if(cur_chan_numb >= chanel_number){
      cur_chan_numb = 0;
      calc_rest = calc_rest + PPM_PulseLen;// 
      OCR1A = (PPM_FrLen - calc_rest) * 2;
      calc_rest = 0;
    }
    else{
      OCR1A = (ppm[cur_chan_numb] - PPM_PulseLen) * 2;
      calc_rest = calc_rest + ppm[cur_chan_numb];
      cur_chan_numb++;
    }     
  }
}

/****************************************************GOES IN LOOP TO INTERPRETE DATA IF GOING BACK TO OLD WAYS***************************************************************************
 *   /*for(int i = 0; i < dataSize; i++){
    incomingData[i] = Serial.read();   //take the incoming data and stores it into an array
    //Serial.println(incomingData[i]);
    //delay(10);
  }
  for(int i = 0; i < 3; i++){
    ppm [5+i] = 1100 + incomingData[1+i]*800;
  }
  if(incomingData[0] == 1 && previousStates[0] == 0){  //if the first byte (the arm disarm byte) is 1, meaning the copter needs to armed, set the ppm values (throttle and yaw) that arm the copter. This is equivalent to holding the joystick to the bottom right (look up youtube
    armCopter();
  }
  else if(incomingData[0] == 0 && previousStates[0] == 1){    //if the copter needs to be disarmed then set the corresponding ppm values to disarm it (again look at youtube video at how it gets disarmed)
    disarmCopter();
  }
  else if(incomingData[0] == 1 && incomingData[0] == previousStates[0]){   //if the copter is already armed then change ppm values
    switch (incomingData[4]){      //if the joy stick was pushed up, then that means increment ppm throttle value by 100 (in this specific case). 100 can be changed to any other number
      case 1:
        for(int i = 0; i < 20; i++){
          ppm[2]++;
          delay(1);
        }
        //ppm[2] += throttleIncrement;
        break;
      case 2:
        for(int i = 0; i < 20; i++){
          ppm[2]--;
          delay(1);
        }
        //ppm[2] -= throttleIncrement;             //if the joy stick was pushed downward, decrease throttle by 100
        break;
      case 3:
        //do nothing               //if the joystick was pushed left or right then keep throttle where it is
        break;  
    }
    switch (incomingData[5]){     //same process as above
      case 1: 
        for(int i = 0; i < 20; i++){
          ppm[3]++;
          delay(1);
        }
        //ppm[3] += throttleIncrement;
        break;
      case 2:
        for(int i = 0; i < 20; i++){
          ppm[3]--;
          delay(1);
        }
        //ppm[3] -= throttleIncrement;
        break;
      case 3:
        //do nothing
        break;
    }
  }
  if(ppm[2] < 1000){ ppm[2] = 1000; }
  else if(ppm[2] > 1500){ ppm[2] = 1500; }
  if(ppm[3] < 1000){ ppm[3] = 1000; }
  else if(ppm[3] > 2000){ ppm[3] = 2000; }
  for(int i = 0; i < numStates; i++){
    previousStates[i] = incomingData[i];
  }
  //Serial.print("throttle is "); Serial.println(ppm[2]);
  //Serial.print("Yaw is "); Serial.println(ppm[3]);
  }
}******************************************************************************************************************/


/***************************************************************
 * Function: converIncomingData
 * Parameters: none
 * Information: takes the incoming data the user has sent from the other arduino and manipulates the ppm values that are being sent to the flight controller
 *//*
void convertIncomingData(){
  for(int i = 0; i < dataSize; i++){
    incomingData[i] = XBee.read();   //take the incoming data and stores it into an array
  }
  for(int i = 0; i < 3; i++){
    ppm1[5+i] = 1100 + incomingData[1+i]*800;
  }
  if(incomingData[0] == 1 && previousStates[0] == 0){  //if the first byte (the arm disarm byte) is 1, meaning the copter needs to armed, set the ppm values (throttle and yaw) that arm the copter. This is equivalent to holding the joystick to the bottom right (look up youtube
    armCopter();
    disarmCopter();
  }
  else if(incomingData[0] == 0 && previousStates[0] == 1){    //if the copter needs to be disarmed then set the corresponding ppm values to disarm it (again look at youtube video at how it gets disarmed)
    disarmCopter();
  }
  else if(incomingData[0] == 1 && incomingData[0] == previousStates[0]){   //if the copter is already armed then change ppm values
    switch (incomingData[4]){      //if the joy stick was pushed up, then that means increment ppm throttle value by 100 (in this specific case). 100 can be changed to any other number
      case 1:
        ppm1[2] += throttleIncrement;
        break;
      case 2:
        ppm1[2] -= throttleIncrement;             //if the joy stick was pushed downward, decrease throttle by 100
        break;
      case 3:
        ppm1[2] += 0;               //if the joystick was pushed left or right then keep throttle where it is
        break;  
    }
    switch (incomingData[5]){     //same process as above
      case 1: 
        ppm1[3] += throttleIncrement;
        break;
      case 2:
        ppm1[3] -= throttleIncrement;
        break;
      case 3:
        ppm1[3] += 0;
        break;
    }
  }
  //check to make sure value never goes below 1000 or above 2000
  if(ppm1[2] < 1000){ ppm1[2] = 1000; }
  else if(ppm1[2] > 1500){ ppm1[2] = 1500; }
  if(ppm1[3] < 1000){ ppm1[3] = 1000; }
  else if(ppm1[3] > 2000){ ppm1[3] = 2000; }
  for(int i = 0; i < numStates; i++){
    previousStates[i] = incomingData[i];
  }
}*/
