/*****************************************************************
XBee_Serial_Passthrough.ino

Set up a software serial port to pass data between an XBee Shield
and the serial monitor.

Hardware Hookup:
  The XBee Shield makes all of the connections you'll need
  between Arduino and XBee. If you have the shield make
  sure the SWITCH IS IN THE "DLINE" POSITION. That will connect
  the XBee's DOUT and DIN pins to Arduino pins 2 and 3.

*****************************************************************/
// We'll use SoftwareSerial to communicate with the XBee:
#include <SoftwareSerial.h>
#include <String.h>
// XBee's DOUT (TX) is connected to digital pin 2 (Arduino's Software RX)
// XBee's DIN (RX) is connected to digital pin 3 (Arduino's Software TX)
SoftwareSerial XBee(2, 3); // RX, TX

//joystick parameteres
#define ROLL 0         //Analog pin 0
#define PITCH 1        //Analog pin 1
#define THROTTLE 2     //Analog pin 2
//variables
int rollValue, pitchValue, throttleValue;
int numChars = 7;
int transmitData[7];
int previousStates[4]; 
int currentStates[4];
int upperLimit = 800;   //potentiometer is built into the joystick so when the pins are hooked up to analog pins the analog read gives values from 0-1023 corresponding to 0-5V from the
int lowerLimit = 200;   //potentionemeter. Upper and lower limit are meant to be the boundaries where the user pushes the joystick far enough to signify a change from the neutral position

void setup()
{
  // Set up both ports at 9600 baud. This value is most important
  // for the XBee. Make sure the baud rate matches the config
  // setting of your XBee.
  XBee.begin(9600);
  Serial.begin(9600);
  //pinMode(7, OUTPUT);
  for(int i = 0; i < numChars; i++){
    transmitData[i] = 0;
  }
  for(int i = 0; i < 4; i++){
    pinMode(i+4, INPUT);
  }
  /*for(int i = 0; i < 4; i++){
    previousStates[i] = digitalRead(i+4);
  }*/
}


void loop()
{  
  getJoystickData();                    //get all current values
  for(int i = 0; i < numChars ; i++){
    Serial.println(transmitData[i]);
    XBee.write(transmitData[i]);        //display data over the serial monitor 
  }
  delay(10);  
}

void getJoystickData(){
    transmitData[0] = map(analogRead(THROTTLE), 0, 1023, 0, 256);
    transmitData[1] = map(analogRead(ROLL), 0, 1023, 0, 256);
    transmitData[2] = map(analogRead(PITCH), 0, 1023, 0, 256);
    for(int i = 0; i < 4; i++){
      transmitData[i+3] = digitalRead(i+4);
    }
}

/**********************************************************************************
 * Function: checkChange
 * parameters: none
 * Information: This function looks for a change in the joystick position from the nuetral position.
 * If the the user has changed the position then the function ends and stores the stores 
 * the values of the joystick position in yawValue (left or right) and throttleValue (up or down on the joystick)
 */

/*void checkChange(){
    rollValue = analogRead(ROLL);           
    pitchValue = analogRead(PITCH);
    throttleValue = analogRead(THROTTLE);
    for(int i = 0; i < 4; i++){
      currentStates[i] = digitalRead(i+4);
    } 
    while(lowerLimit < rollValue && lowerLimit < pitchValue && upperLimit > rollValue && upperLimit > pitchValue && previousStates[0] == currentStates[0] && 
          previousStates[1] == currentStates[1] && previousStates[2] == currentStates[2] && previousStates[3] == currentStates[3]){
      rollValue = analogRead(ROLL);
      pitchValue = analogRead(PITCH);
      throttleValue = analogRead(THROTTLE);
      for(int i = 0; i < 4; i++){
        previousStates[i] = currentStates[i];
        currentStates[i] = digitalRead(i+4);
      }      
    }
    for(int i = 0; i < 4; i++){
       previousStates[i] = currentStates[i];
    }
}*/

/*void getThrottleValue(){
  throttleValue = analogRead(THROTTLE);
  throttleValue[0] = (analogRead(THROTTLE) + 1) - (3*256);
  throttleValue[1] = (analogRead(THROTTLE) + 1) - 
}*/


